{*****************************************************************************}
{  DDObjects                                            www.delphi-online.at  }
{  Copyright (C) 2006 Stefan Meisner, stefan.meisner@gmx.net                  }
{  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS    }
{  OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABI-    }
{  LITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT OF THIRD PARTY  }
{  RIGHTS. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR   }
{  ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,   }
{  TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFT-    }
{  WARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                         }
{*****************************************************************************}

unit DDOBasicRemoteTests;

{$I DDObjectsTests.inc}

interface

uses
  SysUtils, TestFramework, DDOTestExtensions, Classes, DDOSystem,
  DDOTestTypes, DDOPersistentClasses, DDOClient,
  uDDOBasicTestClass_Stub, uDDOBasicTestClass_Proxy,
  uDDOTestRecord_Proxy, uDDOTestRecord_Stub,
  uDDOTestEnum_Proxy, uDDOTestEnum_Stub,
  uDDOPersistentTests_Proxy, uDDOPersistentTests_Stub;

type
  TDDOBasicRemoteTests = class(TDDOBaseRemoteTest)
  private
    FProxy: uDDOBasicTestClass_Proxy.TDDOBasicTestClassProxyStub;
    FCounter: Integer;
    FAsyncGuid: TStrings;
    procedure DoTestCallback(Sender: TObject; Counter: Integer);
    procedure OnAsyncEvent(Sender: TDDOProxyBase; GUID: String;
      Result: Integer);
  protected
    procedure SetUp; override;
  published
    procedure SimpleCallTest;
    procedure OneParamTest;
    procedure OutParamTest;
    procedure VarParamTest;
    procedure ParamsOutTest;
    procedure PerformanceTest; override;
    procedure KeepAliveTest;
    procedure KeepAliveFailTest;
    procedure CallbackTest;
    procedure TestException1;
    procedure TestException2;
    procedure TestException3;
    procedure TestStream;
    procedure TestAsyncCall;
    procedure TestWideString;
    procedure EmptyStringTest;
    {$IFDEF MemoryTests}
    procedure MemoryUsageTest;
    {$ENDIF}
  end;

  TDDORemoteRecordTests = class(TDDOBaseRemoteTest)
  private
    FProxy: uDDOTestRecord_Proxy.TDDOTestRecord;
  protected
    procedure SetUp; override;
  published
    procedure TestRecordIn;
    procedure TestRecordVar;
    procedure TestRecordOut;
    procedure PerformanceTest; override;
    procedure TestRecordEx;
    procedure TestRecordEx2;
  end;

  TDDORemoteSetEnumTests = class(TDDOBaseRemoteTest)
  private
    FProxy: uDDOTestEnum_Proxy.TDDOTestEnum;
  protected
    procedure SetUp; override;
  published
    procedure TestEnumIn;
    procedure TestEnumVar;
    procedure TestEnumOut;
    procedure TestSetIn;
    procedure TestSetVar;
    procedure TestSetOut;
    procedure PerformanceTest; override;
  end;

  TDDORemotePersistentTests = class(TDDOBaseRemoteTest)
  private
    FProxy: uDDOPersistentTests_Proxy.TDDOPersistentTests;
  protected
    procedure SetUp; override;
  public
    class procedure CheckStrings(AStrings: TDDOPersistentStrings);
    class procedure SetStrings(AStrings: TDDOPersistentStrings);
    class procedure CheckList(AList: TDDOPersistentList);
    class procedure SetList(AList: TDDOPersistentList);
  published
    procedure TestSetStrings;
    procedure TestSetList;
    procedure TestGetStrings;
    procedure TestGetList;
    procedure TestNilOutTest;
    procedure PerformanceTest; override;    
  end;

implementation

uses
  Windows, DDOShared, Forms, DDOTimer, DDOExceptions;

const
  CClientValue = 'String send by client with german umlauts ';
  CServerValue = 'Sting returned by server with german umlauts ';
  CTestString = 'hello world. how do you do?';
  CAsyncCalc = 42;

type
  ENotRegistered = class(Exception);

  TDDOBasicTestClass = class(TDDOBasicTestClass_ActiveStub)
  private
    FTimer: TDDOTimer;
    FCounter: Integer;
    procedure DoTimer(Sender: TObject);
  public
    procedure TestMethod1; override;
    procedure TestMethod2(DDOParam0: String); override;
    procedure TestMethod3(out DDOParam0: String); override;
    procedure TestMethod4(var DDOParam0: String); override;
    procedure TestMethod5(DDOParam0: Integer;
      DDOParam1: Integer; out DDOParam2: Integer); override;
    procedure TestMethod6; override;
    procedure TestException1; override;
    procedure TestException2; override;
    procedure TestException3; override;
    procedure SubscriptionChanged; override;
    procedure TestStream(out AStream: TStream); override;
    procedure TestAsync(Count: Integer; out Result: Integer); override;
    procedure TestWideString(paramIn: WideString; out paramOut: WideString); override;    
  public
    procedure Initialize; override;
    destructor Destroy; override;
  end;

  TDDOTestRecord = class(TDDOTestRecord_Stub)
  public
    procedure TestMethodIn(ARecord: DDOTestTypes.TTestRecord); override;
    procedure TestMethodOut(out ARecord: DDOTestTypes.TTestRecord); override;
    procedure TestMethodVar(var ARecord: DDOTestTypes.TTestRecord); override;
    procedure TestRecordEx(var DDOParam0: DDOTestTypes.TTestRecordEx); override;
    procedure TestRecordEx2(var DDOParam0: DDOTestTypes.TTestRecordEx); override;
  end;

  TDDOTestEnum = class(TDDOTestEnum_Stub)
  public
    procedure TestEnumIn(DDOParam0: DDOTestTypes.TTestEnum); override;
    procedure TestEnumVar(var DDOParam0: DDOTestTypes.TTestEnum); override;
    procedure TestEnumOut(out DDOParam0: DDOTestTypes.TTestEnum); override;
    procedure TestSetIn(DDOParam0: DDOTestTypes.TTestEnums); override;
    procedure TestSetVar(var DDOParam0: DDOTestTypes.TTestEnums); override;
    procedure TestSetOut(out DDOParam0: DDOTestTypes.TTestEnums); override;
  end;

  TDDOPersistentTests = class(TDDOPersistentTests_Stub)
  public
    procedure SetStrings(AStrings: TObject); override;
    procedure SetList(Value: TObject); override;
    procedure GetStrings(out DDOParam0: TObject); override;
    procedure GetList(out DDOParam0: TObject); override;
    procedure NilOut(out Value: TObject); override;    
  end;

//----------------------------------------------------------------------------
//2007-01-20 07:35 N,SM:

class procedure TDDORemotePersistentTests.CheckList(
  AList: TDDOPersistentList);
var
  i: Integer;
begin
  for i := 0 to 4 do
    Assert(Integer(AList[i]) = i);
end;

//----------------------------------------------------------------------------
//2007-01-20 07:42 N,SM:

class procedure TDDORemotePersistentTests.CheckStrings(
  AStrings: TDDOPersistentStrings);
var
  i: Integer;
begin
  Assert(AStrings.Duplicates = dupError);
  Assert(AStrings.Sorted = True);
  for i := 0 to 4 do
    Assert(StrToInt(AStrings[i]) = i);
end;

//----------------------------------------------------------------------------
//2007-01-20 07:37 N,SM:

procedure TDDORemotePersistentTests.PerformanceTest;
var
  i: Integer;
begin
  for i := 1 to 500 do
    TestSetStrings;
end;

//----------------------------------------------------------------------------
//2007-01-20 07:46 N,SM:

class procedure TDDORemotePersistentTests.SetList(
  AList: TDDOPersistentList);
var
  i: Integer;
begin
  for i := 0 to 4 do
    AList.Add(Pointer(i));
end;

//----------------------------------------------------------------------------
//2007-01-20 07:42 N,SM:

class procedure TDDORemotePersistentTests.SetStrings(
  AStrings: TDDOPersistentStrings);
var
  i: Integer;
begin
  AStrings.Duplicates := dupError;
  AStrings.Sorted := True;
  for i := 0 to 4 do
    AStrings.Add(IntToStr(i));
end;

//----------------------------------------------------------------------------
//2006-07-02 00:58 N,SM:

procedure TDDOBasicTestClass.SubscriptionChanged;
begin
  inherited;
  FTimer.Enabled := ProxyCount > 0;
end;

//----------------------------------------------------------------------------
//2007-01-08 08:14 N,SM:

destructor TDDOBasicTestClass.Destroy;
begin
  FreeAndNil(FTimer);
  inherited;
end;

//----------------------------------------------------------------------------
//2007-01-08 08:14 N,SM:

procedure TDDOBasicTestClass.DoTimer(Sender: TObject);
begin
  Inc(FCounter);
  TestCallback(FCounter)
end;

//----------------------------------------------------------------------------
//2007-01-08 08:15 N,SM:

procedure TDDOBasicTestClass.Initialize;
begin
  inherited;
  FTimer := TDDOTimer.Create(nil);
  FTimer.OnTimer := DoTimer;
end;

//----------------------------------------------------------------------------
//2007-01-08 08:14 N,SM:

procedure TDDOBasicTestClass.TestMethod1;
begin
end;

//----------------------------------------------------------------------------
//2006-07-02 00:58 N,SM:

procedure TDDOBasicTestClass.TestMethod2(DDOParam0: String);
begin
  Assert(DDOParam0 = CClientValue);
end;

//----------------------------------------------------------------------------
//2006-07-02 00:58 N,SM:

procedure TDDOBasicTestClass.TestMethod3(out DDOParam0: String);
begin
  DDOParam0 := CServerValue;
end;

//----------------------------------------------------------------------------
//2006-07-02 00:58 N,SM:

procedure TDDOBasicTestClass.TestMethod4(var DDOParam0: String);
begin
  Assert((DDOParam0 = CClientValue) or (DDOParam0 = ''));
  DDOParam0 := CServerValue;
end;

//----------------------------------------------------------------------------
//2006-07-02 00:58 N,SM:

procedure TDDOBasicTestClass.TestMethod5(DDOParam0, DDOParam1: Integer;
  out DDOParam2: Integer);
begin
  DDOParam2 := DDOParam0 * DDOParam1;
end;

//----------------------------------------------------------------------------
//2007-01-02 16:38 N,SM:

procedure TDDOBasicRemoteTests.CallbackTest;
begin
  FCounter := 0;
  FProxy.OnTestCallback := DoTestCallback;
  FProxy.Subscribe;
  try
    // wait for some callbacks...
    Sleep(10000);
    CheckGreater(FCounter, 1);
  finally
    FProxy.Unsubscribe;
  end;
end;

//----------------------------------------------------------------------------
//2007-01-02 16:40 N,SM:

procedure TDDOBasicRemoteTests.DoTestCallback(Sender: TObject;
  Counter: Integer);
begin
  FCounter := Counter;
end;

//----------------------------------------------------------------------------
//2006-12-19 01:01 N,SM:

procedure TDDOBasicRemoteTests.KeepAliveFailTest;
begin
  StartExpectingException(EDDOTimeOut);
  Requester.TimeOut := 5000;
  FProxy.TestMethod6;
  StopExpectingException;
end;

//----------------------------------------------------------------------------
//2006-12-19 01:01 N,SM:

procedure TDDOBasicRemoteTests.KeepAliveTest;
begin
  Listener.KeepAlive := True;
  Requester.TimeOut := 5000;
  FProxy.TestMethod6;
end;

//----------------------------------------------------------------------------
//2006-07-02 01:01 N,SM:

{$IFDEF MemoryTests}
procedure TDDOBasicRemoteTests.MemoryUsageTest;
var
  i: integer;
begin
  for i := 1 to 10 do
    SimpleCallTest;
  ExpectedLeak := MemoryStatus.Allocated;
  for i := 1 to 100 do
    SimpleCallTest;
end;
{$ENDIF}

//----------------------------------------------------------------------------
//2006-07-02 03:28 N,SM:

procedure TDDOBasicRemoteTests.OneParamTest;
begin
  FProxy.TestMethod2(CClientValue);
end;

//----------------------------------------------------------------------------
//2006-07-02 01:33 N,SM:

procedure TDDOBasicRemoteTests.OutParamTest;
var
  lValue: String;
begin
  FProxy.TestMethod3(lValue);
  CheckEqualsString(CServerValue, lValue);
end;

//----------------------------------------------------------------------------
//2006-07-02 01:33 N,SM:

procedure TDDOBasicRemoteTests.ParamsOutTest;
var
  i, j, lResult: Integer;
begin
  for i := 1 to 5 do
    for j := 1 to 5 do
    begin
      FProxy.TestMethod5(i, j, lResult);
      CheckEquals(i * j, lResult);
    end;
end;

//----------------------------------------------------------------------------
//2006-07-02 01:33 N,SM:

procedure TDDOBasicRemoteTests.PerformanceTest;
var
  i: Integer;
begin
  for i := 1 to 500 do
    SimpleCallTest;
end;

//----------------------------------------------------------------------------
//2006-07-02 01:34 N,SM:

procedure TDDOBasicRemoteTests.SetUp;
begin
  inherited;
  FProxy := CreateProxy(uDDOBasicTestClass_Proxy.TDDOBasicTestClassProxyStub);
  RegisterClass(TDDOBasicTestClass);
end;

//----------------------------------------------------------------------------
//2006-07-02 01:01 N,SM:

procedure TDDOBasicRemoteTests.SimpleCallTest;
begin
  FProxy.TestMethod1;
end;

//----------------------------------------------------------------------------
//2006-07-02 01:33 N,SM:

procedure TDDOBasicRemoteTests.TestAsyncCall;
var
  i: Integer;
begin
  FAsyncGuid := TDDOThreadStringList.Create;
  try
    for i := 1 to 5 do
      FAsyncGuid.Add(FProxy.TestAsync(CAsyncCalc, OnAsyncEvent));
    // wait for async event (usually one won't wait else the call does
    // not need to be async, but this is for testing purposes...)
    Sleep(10000);
    Assert(FAsyncGuid.Count = 0);
  finally
    FreeAndNil(FAsyncGuid);
  end;
end;

//----------------------------------------------------------------------------
//2007-01-20 18:00 N,SM:

procedure TDDOBasicRemoteTests.OnAsyncEvent(Sender: TDDOProxyBase;
  GUID: String; Result: Integer);
var
  lIdx: Integer;
begin
  lIdx := FAsyncGuid.IndexOf(GUID);
  Assert(lIdx >= 0);
  FAsyncGuid.Delete(lIdx);
  Assert(Result = CAsyncCalc * 2);
end;

//----------------------------------------------------------------------------
//2007-01-20 17:58 N,SM:

procedure TDDOBasicRemoteTests.TestException1;
begin
  StartExpectingException(EOSError);
  FProxy.TestException1;
  StopExpectingException;
end;

//----------------------------------------------------------------------------
//2006-07-02 01:33 N,SM:

procedure TDDOBasicRemoteTests.TestException2;
begin
  StartExpectingException(EAccessViolation);
  FProxy.TestException2;
  StopExpectingException;
end;

//----------------------------------------------------------------------------
//2006-07-02 01:33 N,SM:

procedure TDDOBasicRemoteTests.TestException3;
begin
  StartExpectingException(EDDORemote);
  FProxy.TestException3;
  StopExpectingException;
end;

//----------------------------------------------------------------------------
//2007-01-09 01:07 N,SM:

procedure TDDOBasicRemoteTests.TestStream;
var
  lStream: TStream;
  lString: String;
begin
  FProxy.TestStream(lStream);
  try
    Assert(lStream.Size = Length(CTestString));
    SetLength(lString, Length(CTestString));
    lStream.ReadBuffer(lString[1], Length(CTestString));
    Assert(lString = CTestString);
  finally
    lStream.Free;
  end;
end;

//----------------------------------------------------------------------------
//2007-01-09 01:07 N,SM:

procedure TDDOBasicRemoteTests.VarParamTest;
var
  lValue: String;
begin
  lValue := CClientValue;
  FProxy.TestMethod4(lValue);
  CheckEqualsString(CServerValue, lValue);
end;

//----------------------------------------------------------------------------
//2006-12-19 01:33 N,SM:

procedure TDDOBasicTestClass.TestMethod6;
begin
  Sleep(15000);
end;

//----------------------------------------------------------------------------
//2006-12-27 13:47 N,SM:

procedure TDDORemoteRecordTests.PerformanceTest;
var
  i: Integer;
begin
  for i := 1 to 500 do
    TestRecordIn;
end;

//----------------------------------------------------------------------------
//2006-12-27 13:52 N,SM:

procedure TDDORemoteRecordTests.SetUp;
begin
  inherited;
  FProxy := CreateProxy(uDDOTestRecord_Proxy.TDDOTestRecord);
  RegisterClass(TDDOTestRecord);
end;

//----------------------------------------------------------------------------
//2006-12-27 13:42 N,SM:

procedure TDDOTestRecord.TestMethodIn(ARecord: TTestRecord);
begin
  CompareRecord(ARecord, GetDefaultRecord);
end;

//----------------------------------------------------------------------------
//2006-12-27 13:42 N,SM:

procedure TDDOTestRecord.TestMethodOut(out ARecord: TTestRecord);
begin
  ARecord := GetDefaultRecord;
end;

//----------------------------------------------------------------------------
//2006-12-27 13:42 N,SM:

procedure TDDOTestRecord.TestMethodVar(var ARecord: TTestRecord);
begin
  CompareRecord(ARecord, GetDefaultRecord);
  ARecord := EditRecord(ARecord);
end;

//----------------------------------------------------------------------------
//2006-12-27 13:44 N,SM:

procedure TDDORemoteRecordTests.TestRecordEx;
var
  lRecord: TTestRecordEx;
begin
  lRecord := GetDefaultRecordEx;
  FProxy.TestRecordEx(lRecord);
  CompareRecord(GetDefaultRecordEx, lRecord);
  Assert(lRecord.AVariant = 42);
  Assert(lRecord.ASet = [teValue1, teValue4]);
end;

//----------------------------------------------------------------------------
//2006-12-27 16:28 N,SM:

procedure TDDORemoteRecordTests.TestRecordEx2;
var
  lRecord: TTestRecordEx;
begin
  lRecord := GetDefaultRecordEx;
  FProxy.TestRecordEx2(lRecord);
  CompareRecord(GetDefaultRecordEx, lRecord);
  Assert(lRecord.AVariant = CTestString);
  Assert(lRecord.ASet = [teValue3]);
end;

//----------------------------------------------------------------------------
//2006-12-27 20:42 N,SM:

procedure TDDORemoteRecordTests.TestRecordIn;
var
  lRecord: TTestRecord;
begin
  lRecord := GetDefaultRecord;
  FProxy.TestMethodIn(lRecord);
end;

//----------------------------------------------------------------------------
//2006-12-27 13:44 N,SM:

procedure TDDORemoteRecordTests.TestRecordOut;
var
  lRecord: TTestRecord;
begin
  FProxy.TestMethodOut(lRecord);
  CompareRecord(lRecord, GetDefaultRecord);
end;

//----------------------------------------------------------------------------
//2006-12-27 13:44 N,SM:

procedure TDDORemoteRecordTests.TestRecordVar;
var
  lRecord, lCompare: TTestRecord;
begin
  lRecord := GetDefaultRecord;
  FProxy.TestMethodVar(lRecord);
  lCompare := EditRecord(GetDefaultRecord);
  CompareRecord(lRecord, lCompare);
end;

//----------------------------------------------------------------------------
//2006-12-27 16:26 N,SM:

procedure TDDOTestRecord.TestRecordEx(var DDOParam0: TTestRecordEx);
begin
  CompareRecord(GetDefaultRecordEx, DDOParam0);
  DDOParam0.AVariant := 42;
  DDOParam0.ASet := [teValue1, teValue4];
end;

//----------------------------------------------------------------------------
//2006-12-27 20:44 N,SM:

procedure TDDOTestRecord.TestRecordEx2(var DDOParam0: TTestRecordEx);
begin
  TestRecordEx(DDOParam0);
  DDOParam0.AVariant := CTestString;
  DDOParam0.ASet := [teValue3];
end;

//----------------------------------------------------------------------------
//2007-01-07 19:00 N,SM:

procedure TDDORemoteSetEnumTests.PerformanceTest;
var
  i: Integer;
begin
  for i := 1 to 500 do
  begin
    TestEnumIn;
    TestSetIn;
  end;
end;

//----------------------------------------------------------------------------
//2007-01-07 19:00 N,SM:

procedure TDDORemoteSetEnumTests.SetUp;
begin
  inherited;
  FProxy := CreateProxy(uDDOTestEnum_Proxy.TDDOTestEnum);
  RegisterClass(TDDOTestEnum);
end;

//----------------------------------------------------------------------------
//2007-01-08 02:11 N,SM:

procedure TDDOTestEnum.TestEnumIn(DDOParam0: TTestEnum);
begin
  Assert(DDOParam0 = teValue1);
end;

//----------------------------------------------------------------------------
//2007-01-08 02:11 N,SM:

procedure TDDOTestEnum.TestEnumOut(out DDOParam0: TTestEnum);
begin
  DDOParam0 := teValue3;
end;

//----------------------------------------------------------------------------
//2007-01-08 02:11 N,SM:

procedure TDDOTestEnum.TestEnumVar(var DDOParam0: TTestEnum);
begin
  Assert(DDOParam0 = teValue1);
  DDOParam0 := teValue2;
end;

//----------------------------------------------------------------------------
//2007-01-08 02:11 N,SM:

procedure TDDOTestEnum.TestSetIn(DDOParam0: TTestEnums);
begin
  Assert(DDOParam0 = [teValue1, teValue2]);
end;

//----------------------------------------------------------------------------
//2007-01-08 02:11 N,SM:

procedure TDDOTestEnum.TestSetOut(out DDOParam0: TTestEnums);
begin
  DDOParam0 := [teValue2, teValue3];
end;

//----------------------------------------------------------------------------
//2007-01-08 02:11 N,SM:

procedure TDDOTestEnum.TestSetVar(var DDOParam0: TTestEnums);
begin
  Assert(DDOParam0 = [teValue3, teValue4]);
  DDOParam0 := [teValue1, teValue2];
end;

//----------------------------------------------------------------------------
//2007-01-08 02:20 N,SM:

procedure TDDORemoteSetEnumTests.TestEnumIn;
begin
  FProxy.TestEnumIn(teValue1);
end;

//----------------------------------------------------------------------------
//2007-01-08 02:20 N,SM:

procedure TDDORemoteSetEnumTests.TestEnumOut;
var
  lValue: TTestEnum;
begin
  FProxy.TestEnumOut(lValue);
  Assert(lValue = teValue3);
end;

//----------------------------------------------------------------------------
//2007-01-08 02:20 N,SM:

procedure TDDORemoteSetEnumTests.TestEnumVar;
var
  lValue: TTestEnum;
begin
  lValue := teValue1;
  FProxy.TestEnumVar(lValue);
  Assert(lValue = teValue2);
end;

//----------------------------------------------------------------------------
//2007-01-08 02:25 N,SM:

procedure TDDORemoteSetEnumTests.TestSetIn;
begin
  FProxy.TestSetIn([teValue1, teValue2]);
end;

//----------------------------------------------------------------------------
//2007-01-08 02:25 N,SM:

procedure TDDORemoteSetEnumTests.TestSetOut;
var
  lValue: TTestEnums;
begin
  FProxy.TestSetOut(lValue);
  Assert(lValue = [teValue2, teValue3]);
end;

//----------------------------------------------------------------------------
//2007-01-08 02:25 N,SM:

procedure TDDORemoteSetEnumTests.TestSetVar;
var
  lValue: TTestEnums;
begin
  lValue := [teValue3, teValue4];
  FProxy.TestSetVar(lValue);
  Assert(lValue = [teValue1, teValue2]);
end;

//----------------------------------------------------------------------------
//2007-01-08 21:46 N,SM:

procedure TDDOBasicTestClass.TestException1;
var
  lError: EOSError;
begin
  lError := EOSError.Create(EmptyStr);
  lError.ErrorCode := 42;
  raise lError;
end;

//----------------------------------------------------------------------------
//2007-01-08 21:46 N,SM:

procedure TDDOBasicTestClass.TestException2;
begin
  raise EAccessViolation.Create(EmptyStr);
end;

//----------------------------------------------------------------------------
//2007-01-08 21:46 N,SM:

procedure TDDOBasicTestClass.TestException3;
begin
  raise ENotRegistered.Create(EmptyStr);
end;

//----------------------------------------------------------------------------
//2007-01-09 01:08 N,SM:

procedure TDDOBasicTestClass.TestStream(out AStream: TStream);
var
  lStream: TStringStream;
begin
  lStream := FreeAfterProcessing(TStringStream.Create(EmptyStr));
  lStream.WriteString(CTestString);
  AStream := lStream;
end;

//----------------------------------------------------------------------------
//2007-01-20 07:24 N,SM:

procedure TDDORemotePersistentTests.SetUp;
begin
  inherited;
  FProxy := CreateProxy(uDDOPersistentTests_Proxy.TDDOPersistentTests);
  RegisterClass(TDDOPersistentTests);
end;

//----------------------------------------------------------------------------
//2007-01-20 07:25 N,SM:

procedure TDDORemotePersistentTests.TestGetList;
var
  lList: TDDOPersistentList;
begin
  FProxy.GetList(TObject(lList));
  try
    TDDORemotePersistentTests.CheckList(lList);
  finally
    lList.Free;
  end;
end;

//----------------------------------------------------------------------------
//2007-01-20 07:25 N,SM:

procedure TDDORemotePersistentTests.TestGetStrings;
var
  lStrings: TDDOPersistentStrings;
begin
  FProxy.GetStrings(TObject(lStrings));
  try
    TDDORemotePersistentTests.CheckStrings(lStrings);
  finally
    lStrings.Free;
  end;
end;

//----------------------------------------------------------------------------
//2007-01-20 07:25 N,SM:

procedure TDDORemotePersistentTests.TestNilOutTest;
var
  lTest: TObject;
begin
  FProxy.NilOut(lTest);
  Assert(lTest = nil);
end;

//----------------------------------------------------------------------------
//2007-01-24 19:00 N,SM:

procedure TDDORemotePersistentTests.TestSetList;
var
  lList: TDDOPersistentList;
begin
  lList := TDDOPersistentList.Create;
  try
    TDDORemotePersistentTests.SetList(lList);
    FProxy.SetList(lList);
  finally
    lList.Free;
  end;
end;

//----------------------------------------------------------------------------
//2007-01-20 07:25 N,SM:

procedure TDDORemotePersistentTests.TestSetStrings;
var
  lStrings: TDDOPersistentStrings;
begin
  lStrings := TDDOPersistentStrings.Create;
  try
    TDDORemotePersistentTests.SetStrings(lStrings);
    FProxy.SetStrings(lStrings);
  finally
    lStrings.Free;
  end;
end;

//----------------------------------------------------------------------------
//2007-01-20 07:26 N,SM:

procedure TDDOPersistentTests.GetList(out DDOParam0: TObject);
var
  lList: TDDOPersistentList;
begin
  lList := TDDOPersistentList.Create;
  FreeAfterProcessing(lList);
  TDDORemotePersistentTests.SetList(lList);
  DDOParam0 := lList;
end;


//----------------------------------------------------------------------------
//2007-01-20 07:26 N,SM:

procedure TDDOPersistentTests.GetStrings(out DDOParam0: TObject);
var
  lStrings: TDDOPersistentStrings;
begin
  lStrings := TDDOPersistentStrings.Create;
  FreeAfterProcessing(lStrings);
  TDDORemotePersistentTests.SetStrings(lStrings);
  DDOParam0 := lStrings;
end;

//----------------------------------------------------------------------------
//2007-01-20 07:26 N,SM:

procedure TDDOPersistentTests.NilOut(out Value: TObject);
begin
  Value := nil;
end;

//----------------------------------------------------------------------------
//2007-01-24 19:01 N,SM:

procedure TDDOPersistentTests.SetList(Value: TObject);
var
  lList: TDDOPersistentList;
begin
  lList := Value as TDDOPersistentList;
  TDDORemotePersistentTests.CheckList(lList);
end;

//----------------------------------------------------------------------------
//2007-01-20 07:26 N,SM:

procedure TDDOPersistentTests.SetStrings(AStrings: TObject);
var
  lStrings: TDDOPersistentStrings;
begin
  lStrings := AStrings as TDDOPersistentStrings;
  TDDORemotePersistentTests.CheckStrings(lStrings);
end;

//----------------------------------------------------------------------------
//2007-01-20 18:02 N,SM:

procedure TDDOBasicTestClass.TestAsync(Count: Integer;
  out Result: Integer);
begin
  Result := Count * 2;
end;

//----------------------------------------------------------------------------
//2007-01-20 18:02 N,SM:

procedure TDDOBasicRemoteTests.TestWideString;
var
  i, o: WideString;
begin
  i := '';  // german umlauts
  FProxy.TestWideString(i, o);
  CheckEqualsString(i + i, o);
end;

//----------------------------------------------------------------------------
//2007-01-20 18:02 N,SM:

procedure TDDOBasicTestClass.TestWideString(paramIn: WideString;
  out paramOut: WideString);
begin
  paramOut := paramIn + paramIn;
end;

procedure TDDOBasicRemoteTests.EmptyStringTest;
var
  lValue: String;
begin
  lValue := '';
  FProxy.TestMethod4(lValue);
  CheckEqualsString('', '');
end;


initialization
  TestFramework.RegisterTest('DDORemoteTests',
    TDDOBasicRemoteTests.Suite);
  TestFramework.RegisterTest('DDORemoteTests',
    TDDORemoteSetEnumTests.Suite);
  TestFramework.RegisterTest('DDORemoteTests',
    TDDORemoteRecordTests.Suite);
  TestFramework.RegisterTest('DDORemoteTests',
    TDDORemotePersistentTests.Suite);

end.
