{*****************************************************************************}
{  DDObjects                                            www.delphi-online.at  }
{  Copyright (C) 2006 Stefan Meisner, stefan.meisner@gmx.net                  }
{  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS    }
{  OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABI-    }
{  LITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT OF THIRD PARTY  }
{  RIGHTS. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR   }
{  ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,   }
{  TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFT-    }
{  WARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                         }
{*****************************************************************************}

unit uClientMainForm;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  Dialogs, ExtCtrls, DDOBase, DDOClient, StdCtrls, Spin, uAsyncDemo_Proxy,
  DDODocument, DDOUtils;

const
  UM_DEL = WM_USER +1;
  UM_ADD = WM_USER +2;

type
  TDemoClientMainForm = class(TForm)
    Label1: TLabel;
    EditHost: TEdit;
    EditPort: TSpinEdit;
    ButtonAsync: TButton;
    DDORequester1: TDDORequester;
    Panel1: TPanel;
    MemoRequests: TMemo;
    MemoResults: TMemo;
    Label2: TLabel;
    Label3: TLabel;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure ButtonAsyncClick(Sender: TObject);
    procedure EditHostChange(Sender: TObject);
    procedure EditPortChange(Sender: TObject);
    procedure DDORequester1DataEvent(Sender: TObject; Sending: Boolean;
      var Data: String);
    procedure DDORequester1Exception(Sender: TObject; e: Exception);
    procedure DDORequester1ReceiveData(Sender: TObject; Size, Expected: Integer);
    procedure DDORequester1ResultReceived(Sender: TObject;
      Result: TDDODocument);
    procedure DDORequester1SendRequest(Sender: TObject;
      Request: TDDODocument);
    procedure DDORequester1AsyncRequestFailed(Sender: TObject;
      const GUID: String; E: Exception);
    procedure DDORequester1ExceptionReceived(Sender: TObject;
      e: Exception);
  private
    FProxy: TAsyncDemo;
    procedure GetSomeDataCallback(Sender: TDDOProxyBase;
      GUID: String; Data: String);
    procedure UMAdd(var AMessage: TMessage); message UM_ADD;
    procedure UMDel(var AMessage: TMessage); message UM_DEL;
    procedure AddText(AMemo: TMemo; AString: String);
    procedure DelText(AMemo: TMemo; AString: String);
  end;

var
  DemoClientMainForm: TDemoClientMainForm;

implementation
{$IFNDEF Delphi6up}
uses
  ComObj;
{$ENDIF}

{$R *.dfm}

//----------------------------------------------------------------------------
//2006-02-21 N,SM:

procedure TDemoClientMainForm.FormCreate(Sender: TObject);
begin
  FProxy := TAsyncDemo.Create(DDORequester1);
  EditHost.Text := DDORequester1.Host;
  EditPort.Value := DDORequester1.Port;
end;

//----------------------------------------------------------------------------
//2006-02-21 N,SM:

procedure TDemoClientMainForm.FormDestroy(Sender: TObject);
begin
  FreeAndNil(FProxy);
end;

//----------------------------------------------------------------------------
//2006-02-21 N,SM:

procedure TDemoClientMainForm.EditHostChange(Sender: TObject);
begin
  DDORequester1.Host := EditHost.Text;
end;

//----------------------------------------------------------------------------
//2006-02-21 N,SM:

procedure TDemoClientMainForm.EditPortChange(Sender: TObject);
begin
  DDORequester1.Port := EditPort.Value;
end;

//----------------------------------------------------------------------------
//2006-02-21 N,SM:

procedure TDemoClientMainForm.ButtonAsyncClick(Sender: TObject);
var
  lGUID: String;
begin
  lGUID := FProxy.GetSomeData(Random(99), GetSomeDataCallback);
  AddText(MemoRequests, lGUID);
end;

//----------------------------------------------------------------------------
//2006-02-22 N,SM:

procedure TDemoClientMainForm.GetSomeDataCallback(Sender: TDDOProxyBase;
  GUID: String; Data: String);
begin
  DelText(MemoRequests, GUID);
  AddText(MemoResults, TimeToStr(Now));
  AddText(MemoResults, ' ' + Data);
end;

//----------------------------------------------------------------------------
//2006-02-22 N,SM:

procedure TDemoClientMainForm.DDORequester1DataEvent(Sender: TObject;
  Sending: Boolean; var Data: String);
begin
  AddText(MemoResults, 'DataEvent');
end;

//----------------------------------------------------------------------------
//2006-02-22 N,SM:

procedure TDemoClientMainForm.DDORequester1Exception(Sender: TObject;
  e: Exception);
begin
  AddText(MemoResults, 'ExceptionEvent');
end;

//----------------------------------------------------------------------------
//2006-02-22 N,SM:

procedure TDemoClientMainForm.DDORequester1ReceiveData(Sender: TObject;
  Size, Expected: Integer);
begin
  AddText(MemoResults, 'ReceiveDataEvent');
end;

//----------------------------------------------------------------------------
//2006-02-22 N,SM:

procedure TDemoClientMainForm.DDORequester1ResultReceived(Sender: TObject;
  Result: TDDODocument);
begin
  AddText(MemoResults, 'ResultReceivedEvent');
end;

//----------------------------------------------------------------------------
//2006-02-22 N,SM:

procedure TDemoClientMainForm.DDORequester1SendRequest(Sender: TObject;
  Request: TDDODocument);
begin
  AddText(MemoResults, 'SendRequestEvent');
end;

//----------------------------------------------------------------------------
//2006-02-23 N,SM:

procedure TDemoClientMainForm.DDORequester1AsyncRequestFailed(
  Sender: TObject; const GUID: String; E: Exception);
begin
  GetSomeDataCallback(nil, GUID, E.Message);
end;

//----------------------------------------------------------------------------
//2007-01-20 19:52 N,SM:

procedure TDemoClientMainForm.DDORequester1ExceptionReceived(
  Sender: TObject; e: Exception);
begin
  AddText(MemoResults, 'ExceptionReceived');
end;

//----------------------------------------------------------------------------
//2007-01-20 19:53 N,SM:

procedure TDemoClientMainForm.AddText(AMemo: TMemo; AString: String);
begin
  // This forces a thread switch to the MainThread
  SendMessage(Handle, UM_ADD, Integer(AMemo), Integer(PChar(AString)));
end;

//----------------------------------------------------------------------------
//2007-01-21 01:47 N,SM:

procedure TDemoClientMainForm.DelText(AMemo: TMemo; AString: String);
begin
  // This forces a thread switch to the MainThread
  SendMessage(Handle, UM_DEL, Integer(AMemo), Integer(PChar(AString)));
end;

//----------------------------------------------------------------------------
//2007-01-21 01:51 N,SM:

procedure TDemoClientMainForm.UMAdd(var AMessage: TMessage);
var
  lMemo: TMemo;
  lString: String;
begin
  lMemo := TMemo(AMessage.WParam);
  lString := String(AMessage.LParam);
  lMemo.Lines.Add(lString);
end;

//----------------------------------------------------------------------------
//2007-01-21 01:51 N,SM:

procedure TDemoClientMainForm.UMDel(var AMessage: TMessage);
var
  lMemo: TMemo;
  lString: String;
  lIdx: Integer;
begin
  lMemo := TMemo(AMessage.WParam);
  lString := String(AMessage.LParam);
  lIdx := lMemo.Lines.IndexOf(lString);
  Assert(lIdx >= 0);
  lMemo.Lines.Delete(lIdx);
end;

end.
