{*****************************************************************************}
{  DDObjects                                            www.delphi-online.at  }
{  Copyright (C) 2006 Stefan Meisner, stefan.meisner@gmx.net                  }
{  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS    }
{  OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABI-    }
{  LITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT OF THIRD PARTY  }
{  RIGHTS. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR   }
{  ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,   }
{  TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFT-    }
{  WARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                         }
{*****************************************************************************}

unit uClientMainForm;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  Dialogs, Buttons, DDOClient,  StdCtrls, DDOShared, DDOConsts,
  DDOServer, DDOClientEx, ExtCtrls, uClientCallback_Proxy, DDOBase;

type
  TClientCallback = class;

  TClientMainForm = class(TForm)
    Panel1: TPanel;
    Panel2: TPanel;
    MemoNotifications: TMemo;
    DDORequester1: TDDORequester;
    SpeedButtonLogin: TSpeedButton;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure SpeedButtonLoginClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
  private
    FCallback: TClientCallback;
    procedure DoNotification(Sender: TObject; DateTime: TDateTime; Data: String);
    procedure UpdateGUI;
  end;

  TClientCallback = class(TClientCallbackProxyStub)
  end;

var
  ClientMainForm: TClientMainForm;

implementation

{$R *.dfm}

//----------------------------------------------------------------------------
//2006-02-15 N,SM:

procedure TClientMainForm.FormCreate(Sender: TObject);
begin
  FCallback := TClientCallback.Create(DDORequester1, False);
  FCallback.OnNotification := DoNotification;
end;

//----------------------------------------------------------------------------
//2006-02-15 N,SM:

procedure TClientMainForm.FormDestroy(Sender: TObject);
begin
  FreeAndNil(FCallback);
end;

//----------------------------------------------------------------------------
//2006-02-15 N,SM:

procedure TClientMainForm.SpeedButtonLoginClick(Sender: TObject);
begin
  try
    if not FCallback.Subscribed then
      FCallback.Subscribe
    else
      FCallback.Unsubscribe;
  finally
    UpdateGUI;
  end;
end;

//----------------------------------------------------------------------------
//2006-02-15 N,SM:

procedure TClientMainForm.DoNotification(Sender: TObject;
  DateTime: TDateTime; Data: String);
begin
  // Setting Memo.Text does not need to be synchronized with the
  // MainThread as it does use SendMessage internally which does
  // force a thread context switch.
  MemoNotifications.Text := DateTimeToStr(DateTime) + #13#10 + Data;
end;

//----------------------------------------------------------------------------
//2006-02-15 N,SM:

procedure TClientMainForm.FormClose(Sender: TObject;
  var Action: TCloseAction);
begin
  Action := caFree;
end;

//----------------------------------------------------------------------------
//2006-03-01 13:12 N,SM:
//2006-03-01 13:12 R,SM:

procedure TClientMainForm.UpdateGUI;
begin
  SpeedButtonLogin.Down := FCallback.Subscribed;
end;

end.
