{*****************************************************************************}
{  DDObjects                                            www.delphi-online.at  }
{  Copyright (C) 2006 Stefan Meisner, stefan.meisner@gmx.net                  }
{  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS    }
{  OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABI-    }
{  LITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT OF THIRD PARTY  }
{  RIGHTS. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR   }
{  ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,   }
{  TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFT-    }
{  WARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                         }
{*****************************************************************************}

unit uDemoClient;

interface

uses
  DDOPersistent, Classes;

type
  TDemoClient = class(TInterfacedObject, IDDOPersistent)
  private
    FID: Integer;
    FName,
    FEMail,
    FURL: String;
    FInfo: TStrings;
    procedure SetInfo(Value: TStrings);
  protected
    // IDDOPersistent
    procedure DDOLoadFromStream(Stream: TStream);
    procedure DDOSaveToStream(Stream: TStream);
  public
    constructor Create(ID: Integer);
    destructor Destroy; override;
    procedure Assign(Src: TDemoClient);
    property ID: Integer read FID;
    property Name: String read FName write FName;
    property EMail: String read FEMail write FEMail;
    property URL: String read FURL write FURL;
    property Info: TStrings read FInfo write SetInfo;
  end;

  function DemoClientCreate: TObject;

implementation

uses
  SysUtils, DDOUtils;

//----------------------------------------------------------------------------
//2006-02-20 N,SM:

function DemoClientCreate: TObject;
begin
  Result := TDemoClient.Create(0);
end;

//----------------------------------------------------------------------------
//2006-02-20 N,SM:

procedure TDemoClient.Assign(Src: TDemoClient);
begin
  CopyDDOPersist(Src, Self);
end;

//----------------------------------------------------------------------------
//2006-02-20 N,SM:

constructor TDemoClient.Create(ID: Integer);
begin
  inherited Create;
  FInfo := TStringList.Create;
  FID := ID;
end;

//----------------------------------------------------------------------------
//2006-02-20 N,SM:

destructor TDemoClient.Destroy;
begin
  FreeAndNil(FInfo);
  inherited;
end;

//----------------------------------------------------------------------------
//2006-02-20 N,SM:

procedure TDemoClient.DDOLoadFromStream(Stream: TStream);
begin
  with TReader.Create(Stream, 1024) do
  try
    FID := ReadInteger;
    Name := ReadString;
    EMail := ReadString;
    URL := ReadString;
    Info.Text := ReadString;
  finally
    Free;
  end;
end;

//----------------------------------------------------------------------------
//2006-02-20 N,SM:

procedure TDemoClient.DDOSaveToStream(Stream: TStream);
begin
  with TWriter.Create(Stream, 1024) do
  try
    WriteInteger(ID);
    WriteString(Name);
    WriteString(EMail);
    WriteString(URL);
    WriteString(Info.Text);
  finally
    Free;
  end;
end;

//----------------------------------------------------------------------------
//2006-02-20 N,SM:

procedure TDemoClient.SetInfo(Value: TStrings);
begin
  FInfo.Assign(Value);
end;

initialization
  DDOPersistentRegistry.RegisterClass(TDemoClient, @DemoClientCreate);

finalization
  //empty

end.
 
